#!/bin/bash

# did it work?

# The initial setup of the neural network of pisces 1D. This is run once, at the start of the experiment.

# 1. Makes EXPID directory and copy all the content from EXP00 into a default folder (.xml, executable, namelists)
# 2. Reads the inputted latitude and longitude and makes 3*3 cropped netcdf files from the inputs, and places them in
#    a new directory
# 3. Run the simulation with the default parameters
# 4. Obtaining the bias between the model in default settings and the observed data, to be used as a reference for the
#    other simulations
# 5. Generates an array of randomly assigned parameters as an initial set and saves as a .csv file

## EDIT HERE TO CHANGE THE LOCATION OF THE 1DPISCES
## DEFAULT: LAT=56, LON=-49 (LABRADOR SEA)
LAT=56
LON=-49

SCRHOME=/gpfs/scratch/bsc32/bsc32786/                                                        # base directory of the nemo model
MODEL=${SCRHOME}%EXPID%/EXP00/                                                               # path that will be created and the experiment stored
INPUTS=${SCRHOME}pisces1d/ORCA2_OFF_PISCES/input/                                            # path to input (netCDF) files (PATH ONLY)
NEMO=${SCRHOME}pisces1d/ORCA2_OFF_PISCES/release-4.0/cfgs/PISCES1D/EXP00/                    # path to .xml, executable, and namelist files (PATH ONLY)
OBS=${SCRHOME}pisces1d/ORCA2_OFF_PISCES/obs/Mprof_6901486_L31_month_2015-01-01_2015-12-31.nc # path to the observed data. (PATH AND FILE) MUST BE ON THE SAME GRID AS THE MODEL
VARIABLES=${SCRHOME}pisces1d/ORCA2_OFF_PISCES/release-4.0/cfgs/PISCES1D/EXP00/variables.csv  # path to the information about the variables (file must be called variables.csv) (PATH AND FILE)
MODELFILE=PISCES_1m_19600101_19691231_ptrc_T.nc                                            # name of output file (FILE ONLY)

### 1. MAKE EXPID DIRECTORY AND COPY ALL THE CONTENT FROM EXP00 INTO A DEFAULT FOLDER (.XML, EXECUTABLES, AND NAMELISTS)

mkdir -p ${MODEL}
cd ${MODEL}

mkdir -p bbp700
mkdir -p chla

rm -rf *
mkdir -p default
cd default
cp -R ${NEMO}nemo .
cp -R ${NEMO}namelist* .
cp -R ${NEMO}*.xml .
cd ..
cp ${VARIABLES} .

cd ${INPUTS}

# load appropriate modules
module load python
module load intel/2017.4
module load impi/2017.4
module load netcdf/4.4.1.1
module load hdf5/1.8.19

module load perl
module load udunits/2.2.25
module load gsl/2.4
module load nco

### 2. Reads the inputted latitude and longitude and makes 3*3 cropped netcdf files from the inputs, and places them in
### a new directory

function getindices {           # function that finds the nearest gridpoint to the entered coordinates, and returns
                                # their indices.
inds="$(python - $1 $2 << END

import numpy as np
from geopy import distance
import sys
from netCDF4 import Dataset
import os

def mindistcoords(lat, lon, latitudes, longitudes):
# finds the nearest index
    mindist = 20000
    minI = 0
    minJ = 0
    (x, y) = np.shape(latitudes)
    for I in range(len(latitudes[0])):
        for J in range(len(latitudes[:, 0])):
            if abs(lat - latitudes[J][I]) < 2 and abs(lon - longitudes[J][I]) < 2:
                dist = distance.distance((lat, lon), (latitudes[J][I], longitudes[J][I]))
                if dist < mindist:
                    mindist = dist
                    minI = I
                    minJ = J
    return minJ, minI

# variable grid can be any orca2 grid with T (centre of the cell)
grid = Dataset('dyna_grid_T.nc', 'r', format='NETCDF4_CLASSIC')
lons = np.array(grid.variables['nav_lon'])
lats = np.array(grid.variables['nav_lat'])

ilt,iln=mindistcoords(float(sys.argv[1]),float(sys.argv[2]),lats,lons)

print(ilt)
print(iln)

END
)"
}

getindices $LAT $LON

inds=(${inds})

# makes a new directory, crops the netcdf files and puts them there. If it already exists, it is replaced

cd ${INPUTS}
mkdir -p ./cropped

if find "./cropped" -mindepth 1 -print -quit 2>/dev/null | grep -q .; then
    rm ./cropped/*
fi

for file in *.nc
do
  echo cropping $file
  ncks -d x,$((inds[0]-1)),$((inds[0]+1)) -d y,$((inds[1]-1)),$((inds[1]+1)) ${file} cropped/${file}
done

cp -R ./cropped/* ${MODEL}default
mkdir ${MODEL}cropped
cp -R ./cropped/* ${MODEL}cropped

### 3.  run the simulation with the default parameters

cd ${MODEL}default
module load netcdf/4.2
mpirun -np 1 ./nemo

### 4. Obtaining the bias between the model in default settings and the observed data, to be used as a reference for the
###    other simulations

function getstats {

stats="$(python - $1 $2 << END
# $1 - path to the observed data, $2 - path to outputs

from netCDF4 import Dataset
import numpy as np
import os
import math

# extracting the observed data

def getobsdata(ds,varname):
    A = np.array(obs.variables[varname])
    A[A == 0.0] = np.nan
    A=np.log10(A)
    return A

obs=Dataset('$1')
bbp700obs = getobsdata(obs,'bbp700_despiked')
bPOCobs = getobsdata(obs,'bbp700_spike')
chlaobs = getobsdata(obs,'chla_cal')
depths=np.array(obs.variables['deptht'])
obs.close()

# extracting the outputs of the default simulation
mod = Dataset('$2')
bbp700mod = (np.array(mod.variables['POC'])[-60:, :, 1, 1]+np.array(mod.variables['PHY'])[-60:, :, 1, 1])*0.001
bPOCmod = (np.array(mod.variables['GOC'])[-60:, :, 1, 1]+np.array(mod.variables['PHY2'])[-60:, :, 1, 1])*0.001
chlamod = (np.array(mod.variables['NCHL'])[-60:, :, 1, 1]+np.array(mod.variables['DCHL'])[-60:, :, 1, 1])
mod.close()

def editmoddata(arr,d):
    arr=arr[:,:np.shape(d)[0]]
    for i in range(12):
        arr[i] = np.mean(arr[i::12],0)
    arr = arr[:12]
    arr[arr == 200000039192547557376] = np.nan
    arr=np.log10(arr)
    return arr

bbp700mod=editmoddata(bbp700mod,depths)
bPOCmod=editmoddata(bPOCmod,depths)
chlamod=editmoddata(chlamod,depths)

#def gradperiod(A,d):
#    A=np.concatenate(([A[-1]],A,[A[0]]),axis=0)
#    B=np.gradient(A,1,d)
#    return B[0][1:-1]#+B[0][1:-1]
#
#bbp700obs=gradperiod(bbp700obs,depths)[:,10:]
#bbp700mod=gradperiod(bbp700mod,depths)[:,10:]
#chlaobs=gradperiod(chlaobs,depths)[:,:17]
#chlamod=gradperiod(chlamod,depths)[:,:17]

bbp700obs=bbp700obs[:,10:]
bbp700mod=bbp700mod[:,10:]
bPOCobs=bPOCobs[:,10:]
bPOCmod=bPOCmod[:,10:]
chlaobs=chlaobs[:,:17]
chlamod=chlamod[:,:17]

# standardising the data
bbp700std=np.nanstd(bbp700obs)
bbp700mn=np.nanstd(bbp700obs)
bbp700obs=(bbp700obs-bbp700mn)/bbp700std

bPOCstd=np.nanstd(bPOCobs)
bPOCmn=np.nanstd(bPOCobs)
bPOCobs=(bPOCobs-bPOCmn)/bPOCstd

chlastd=np.nanstd(chlaobs)
chlamn=np.nanstd(chlaobs)
chlaobs=(chlaobs-chlamn)/chlastd

bbp700bias = np.nanmean(bbp700mod - bbp700obs)
bPOCbias = np.nanmean(bPOCmod - bPOCobs)
chlabias = np.nanmean(chlamod - chlaobs)

np.savetxt('bbp700refbias.csv',[bbp700bias])
np.savetxt('bPOCrefbias.csv',[bPOCbias])
np.savetxt('chlarefbias.csv',[chlabias])

# small poc stats
mstd=np.nanstd(bbp700mod)/np.nanstd(bbp700obs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bbp700obs.flatten()), np.ma.masked_invalid(bbp700mod.flatten()))[0,1])
RMSE=math.sqrt(np.nansum((bbp700obs-bbp700mod)**2)/bbp700obs.size)
np.savetxt('bbp700stdcorrrmse.csv',[mstd,R,RMSE])

# large poc stats
mstd=np.nanstd(bPOCmod)/np.nanstd(bPOCobs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bPOCobs.flatten()), np.ma.masked_invalid(bPOCmod.flatten()))[0,1])
RMSE=math.sqrt(np.nansum((bPOCobs-bPOCmod)**2)/bPOCobs.size)
np.savetxt('bPOCstdcorrrmse.csv',[mstd,R,RMSE])

END
)"
}

getstats $OBS ${MODEL}default/$MODELFILE

### 5. Generates an array of randomly assigned parameters as an initial set and saves as a .csv file

function makeparameters {

bias="$(python - $1 $2 << END

import numpy as np
import random
import pandas

vars=pandas.read_csv('$2')

A=np.zeros((int($1),len(vars)))

for i in range(np.shape(A)[0]):
    for j in range(len(vars)):
        A[i,j]=random.random()*(vars['max'][j]-vars['min'][j])+vars['min'][j]
    A[0]=vars['defaults']
np.savetxt('0000.csv',A)

END
)"
}

cd $MODEL

mkdir -p bbp700
mkdir -p chla
mkdir -p parameters
cd parameters
if find . -mindepth 1 -print -quit 2>/dev/null | grep -q .; then
    rm *
fi

makeparameters %NUMCHUNKS% ${VARIABLES}
